#!/bin/bash -e

# Script for cleanup after rolling back a snapshots
# Rollback of a snapshot with RollbackLocal.sh will left behind snapshots marked for deletion.
# These snapshots can only be deleted after a reboot. This script can be executed after reboot in order to delete these snapshots.
#
# In oder to adapt this script for other installations:
# 1. Change the variables to fit for you installations
# 2. Check for the btrfs snapshots after installation (btrfs subvolume list /)
# 		If there are other snapshots than @, @home and /var/lib/machines, you have to change the handling of the snapshots further below

errorecho() { cat <<< "$@" 1>&2; }

#
# Variables
#
TEMPMOUNTDIR="/mnt/temp"				# Mount point of the btrfs root
SNAPSHOTDIR="${TEMPMOUNTDIR}/snapshots"	# Directory where snapshots are saved (usually under the btrs root)
DEVICE="/dev/sda6"						# The device (HDD) used
DELETEFLAG="DEL"

#
# Check for root
#
if [ "$(id -u)" != "0" ]
then
    errorecho "ERROR: This script has to be run as root!"
    exit 1
fi

#
# Check if all dirs already exist
#
if [ ! -e "${TEMPMOUNTDIR}" ]
then
	mkdir -p "${TEMPMOUNTDIR}"
fi

#
# Mount device
#
mount "${DEVICE}" "{TEMPMOUNTDIR}"
cd "${TEMPMOUNTDIR}"
sync

if [ -e "${SNAPSHOTDIR}/@home_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/@home_${DELETEFLAG}"
else
	errorecho "ERROR: Cannot delte subvolume ${SNAPSHOTDIR}/@home_${DELETEFLAG}!"
fi

if [ -e "${SNAPSHOTDIR}/machines_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/machines_${DELETEFLAG}"
else
	errorecho "ERROR: Cannot delete subvolume ${SNAPSHOTDIR}/machines_${DELETEFLAG}!"
fi

if [ -e "${SNAPSHOTDIR}/@_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/@_${DELETEFLAG}"
else
	errorecho "ERROR: Cannot delete subvolume ${SNAPSHOTDIR}/@_${DELETEFLAG}!"
fi

#
# Unmount
#
cd ~
umount "${TEMPMOUNTDIR}"