#!/bin/bash -e

# Script for rollback of local snapshot
# Usage RollbackLocal.sh SnapshotName
#
# In oder to adapt this script for other installations:
# 1. Change the variables to fit for you installations
# 2. Check for the btrfs snapshots after installation (btrfs subvolume list /)
# 		If there are other snapshots than @, @home and /var/lib/machines, you have to change the handling of the snapshots further below

errorecho() { cat <<< "$@" 1>&2; }

#
# Variables
#
TEMPMOUNTDIR="/mnt/temp"				# Mount point of the btrfs root
SNAPSHOTDIR="${TEMPMOUNTDIR}/snapshots"	# Directory where snapshots are saved (usually under the btrs root)
DEVICE="/dev/sda6"						# The device (HDD) used
BACKUPDESC=$1
DELETEFLAG="DEL"

#
# Check if parameter given
#
if [ $# != "1" ]
then
    errorecho "ERROR: No backup name to rollback given!"
	errorecho "Usage: LocalRollback.sh BackupName"
    exit 1
fi

#
# Check for root
#
if [ "$(id -u)" != "0" ]
then
    errorecho "ERROR: This script has to be run as root!"
    exit 1
fi

#
# Check if all dirs already exist
#
if [ ! -e "${TEMPMOUNTDIR}" ]
then
	mkdir -p "${TEMPMOUNTDIR}"
fi

#
# Mount device
#
mount "${DEVICE}" "${TEMPMOUNTDIR}"
cd "${TEMPMOUNTDIR}"
sync

#
# Check if backup exists
#
if [[ ! -e "${SNAPSHOTDIR}/${BACKUPDESC}/@_${BACKUPDESC}" || ! -e "${SNAPSHOTDIR}/${BACKUPDESC}/@home_${BACKUPDESC}" || ! -e "${SNAPSHOTDIR}/${BACKUPDESC}/machines_${BACKUPDESC}" ]]
then
	errorecho "ERROR: Cannot find backup ${BACKUPDESC}!"
	cd ~
	umount "${TEMPMOUNTDIR}"
	exit 1
fi

#
# Check for old snapshots to be deleted.
#
if [[ -e "${SNAPSHOTDIR}/@home_${DELETEFLAG}" || -e "${SNAPSHOTDIR}/machines_${DELETEFLAG}" ||  -e "${SNAPSHOTDIR}/@_${DELETEFLAG}" ]]
then
	echo "Found old snapshots. These will be deleted."
fi

if [ -e "${SNAPSHOTDIR}/@home_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/@home_${DELETEFLAG}"
fi

if [ -e "${SNAPSHOTDIR}/machines_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/machines_${DELETEFLAG}"
fi

if [ -e "${SNAPSHOTDIR}/@_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/@_${DELETEFLAG}"
fi

#
# Rollback snapshots
#
mv "${TEMPMOUNTDIR}/@" "${SNAPSHOTDIR}/@_${DELETEFLAG}"
mv "${SNAPSHOTDIR}/${BACKUPDESC}/@_${BACKUPDESC}" "${TEMPMOUNTDIR}/@"

btrfs subvolume delete /var/lib/machines
mv "${SNAPSHOTDIR}/${BACKUPDESC}/machines_${BACKUPDESC}" "${SNAPSHOTDIR}/machines"
rm -r /mnt/temp/@/var/lib/machines
btrfs subvolume snapshot "${SNAPSHOTDIR}/machines" "${TEMPMOUNTDIR}/@/var/lib"
mv "${SNAPSHOTDIR}/machines"  "${SNAPSHOTDIR}/machines_${DELETEFLAG}"

mv "${TEMPMOUNTDIR}/@home" "${SNAPSHOTDIR}/@home_${DELETEFLAG}"
mv "${SNAPSHOTDIR}/${BACKUPDESC}/@home_${BACKUPDESC}" "${TEMPMOUNTDIR}/@home"

#
# Take snapshots of the rolled back state, otherwise the rolled snapshots would vanish
#
btrfs subvolume snapshot "${TEMPMOUNTDIR}/@" "${SNAPSHOTDIR}/${BACKUPDESC}/@_${BACKUPDESC}"
btrfs subvolume snapshot "${TEMPMOUNTDIR}/@home" "${SNAPSHOTDIR}/${BACKUPDESC}/@home_${BACKUPDESC}"
btrfs subvolume snapshot "${TEMPMOUNTDIR}/@/var/lib/machines" "${SNAPSHOTDIR}/${BACKUPDESC}/machines_${BACKUPDESC}"

#
# Unmount
#
cd ~
umount "${TEMPMOUNTDIR}"

echo "Done - reboot the system now!"
echo "After reboot execute RollbackCleanup.sh to delete the old subvolumes".