#!/bin/bash -e

# Script for rollback of remote snapshots created by SnapshotRemote.sh
# Usage: RollbackRemote.sh SnapshotName
#
# In oder to adapt this script for other installations:
# 1. Change the variables to fit for you installations
# 2. Check for the btrfs snapshots after installation (btrfs subvolume list /)
# 		If there are other snapshots than @, @home and /var/lib/machines, you have to change the handling of the snapshots further below

errorecho() { cat <<< "$@" 1>&2; }

#
# Variables
#
TEMPMOUNTDIR="/mnt/temp"						# Mount point of the btrfs root
SNAPSHOTDIR="${TEMPMOUNTDIR}/snapshots"			# Directory where snapshots are saved (usually under the btrs root)
DEVICE="/dev/sda6"								# The device (HDD) used
REMOTEBACKUPDIR="/mnt/Share/Backups"			# Directory where backups are stored (this has to be a already mounted network share)
BACKUPDESC=$1
DELETEFLAG="DEL"
ROLLBACKFLAG="ROLLBACK"

#
# Check if parameter given
#
if [ $# != "1" ]
then
    errorecho "ERROR: No backup name to rollback given!"
	errorecho "Usage: RemoteRollback.sh BackupName"
    exit 1
fi

#
# Check for root
#
if [ "$(id -u)" != "0" ]
then
    errorecho "ERROR: This script has to be run as root!"
    exit 1
fi

#
# Check if Windows share is mounted
#
if [ ! -e "${REMOTEBACKUPDIR}" ]
then
	errorecho "ERROR: Cannot access remote backup directory ${REMOTEBACKUPDIR}!"
	errorecho "Check if the Windows share is mounted."
	exit 1
fi

#
# Check for backups in Windows share
#
if [[ ! -e "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_root_${BACKUPDESC}.tar.gz" || ! -e "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_home_${BACKUPDESC}.tar.gz" || ! -e "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_machines_${BACKUPDESC}.tar.gz" ]]
then
	errorecho "ERROR: Cannot find backup ${BACKUPDESC}!"
	exit 1
fi

#
# Mount device
#
mount ${DEVICE} ${TEMPMOUNTDIR}
cd "${TEMPMOUNTDIR}"
sync

#
# Check if all dirs already exist
#
if [ ! -e "${SNAPSHOTDIR}" ]
then
	mkdir -p "${SNAPSHOTDIR}"
fi

#
# Create empty subvolumes for rollback
#
btrfs subvolume create "${SNAPSHOTDIR}/@_${ROLLBACKFLAG}"
btrfs subvolume create "${SNAPSHOTDIR}/@home_${ROLLBACKFLAG}"
btrfs subvolume create "${SNAPSHOTDIR}/machines_${ROLLBACKFLAG}"

#
# Untar
#
tar xpzf "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_root_${BACKUPDESC}.tar.gz" -C "${SNAPSHOTDIR}/@_${ROLLBACKFLAG}" --strip=4
tar xpzf "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_home_${BACKUPDESC}.tar.gz" -C "${SNAPSHOTDIR}/@home_${ROLLBACKFLAG}" --strip=4
tar xpzf "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_machines_${BACKUPDESC}.tar.gz" -C "${SNAPSHOTDIR}/machines_${ROLLBACKFLAG}" --strip=4

#
# Rollback
#
mv "${TEMPMOUNTDIR}/@" "${SNAPSHOTDIR}/@_${DELETEFLAG}"
mv "${SNAPSHOTDIR}/@_${ROLLBACKFLAG}" "${TEMPMOUNTDIR}/@"

btrfs subvolume delete /var/lib/machines
mv "${SNAPSHOTDIR}/machines_${ROLLBACKFLAG}" "${SNAPSHOTDIR}/machines"
rm -r /mnt/temp/@/var/lib/machines
btrfs subvolume snapshot "${SNAPSHOTDIR}/machines" /mnt/temp/@/var/lib
mv "${SNAPSHOTDIR}/machines" "${SNAPSHOTDIR}/machines_${DELETEFLAG}"

mv "${TEMPMOUNTDIR}/@home" "${SNAPSHOTDIR}/@home_${DELETEFLAG}"
mv "${SNAPSHOTDIR}/@home_${ROLLBACKFLAG}" "${TEMPMOUNTDIR}/@home"

#
# Unmount
#
cd ~
umount "${TEMPMOUNTDIR}"

echo "Done - reboot the system now!"
echo "After reboot execute RollbackCleanup.sh to delete the old subvolumes".