#!/bin/bash -e

# Script for creating local snapshots
# Usage SnapshotLocal.sh SnapShotName
#
# In oder to adapt this script for other installations:
# 1. Change the variables to fit for you installations
# 2. Check for the btrfs snapshots after installation (btrfs subvolume list /)
# 		If there are other snapshots than @, @home and /var/lib/machines, you have to change the handling of the snapshots further below

errorecho() { cat <<< "$@" 1>&2; }

#
# Variables
#
TEMPMOUNTDIR="/mnt/temp"				# Mount point of the btrfs root
SNAPSHOTDIR="${TEMPMOUNTDIR}/snapshots"	# Directory where snapshots are saved (usually under the btrs root)
DEVICE="/dev/sda6"						# The device (HDD) used
BACKUPDESC=$1
DELETEFLAG="DEL"

#
# Check if parameter given
#
if [ $# != "1" ]
then
    errorecho "ERROR: No backup name given!"
	errorecho "Usage: SnapshotLocal.sh BackupName"
    exit 1
fi

#
# Check for root
#
if [ "$(id -u)" != "0" ]
then
	errorecho "ERROR: This script has to be run as root!"
	exit 1
fi

#
# Check if all mount dir already exists
#
if [ ! -e "${TEMPMOUNTDIR}" ]
then
	mkdir -p "${TEMPMOUNTDIR}"
fi

#
# Mount device
#
mount "${DEVICE}" "${TEMPMOUNTDIR}"
cd "${TEMPMOUNTDIR}"
sync

#
# Check if all snapshot dir already exists
#
if [ ! -e "${SNAPSHOTDIR}" ]
then
	mkdir -p "${SNAPSHOTDIR}"
fi

#
# Check for old snapshots to be deleted.
#
if [[ -e "${SNAPSHOTDIR}/@home_${DELETEFLAG}" || -e "${SNAPSHOTDIR}/machines_${DELETEFLAG}" ||  -e "${SNAPSHOTDIR}/@_${DELETEFLAG}" ]]
then
	echo "Found old snapshots. These will be deleted."
fi

if [ -e "${SNAPSHOTDIR}/@home_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/@home_${DELETEFLAG}"
fi

if [ -e "${SNAPSHOTDIR}/machines_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/machines_${DELETEFLAG}"
fi

if [ -e "${SNAPSHOTDIR}/@_${DELETEFLAG}" ]
then
	btrfs subvolume delete "${SNAPSHOTDIR}/@_${DELETEFLAG}"
fi

#
# Check if a backup with the specified name already exists
#
if [[ -e "${SNAPSHOTDIR}/${BACKUPDESC}/@_${BACKUPDESC}" || -e "${SNAPSHOTDIR}/${BACKUPDESC}/@home_${BACKUPDESC}" || -e "${SNAPSHOTDIR}/${BACKUPDESC}/var_lib_machines_${BACKUPDESC}" ]]
then
	errorecho "ERROR: Backup ${BACKUPDESC} already exists!"
	cd ~
	umount "${TEMPMOUNTDIR}"
	exit 1
fi

#
# Create snapshots
#
mkdir "${SNAPSHOTDIR}/${BACKUPDESC}"
btrfs subvolume snapshot "${TEMPMOUNTDIR}/@" "${SNAPSHOTDIR}/${BACKUPDESC}/@_${BACKUPDESC}"
btrfs subvolume snapshot "${TEMPMOUNTDIR}/@home" "${SNAPSHOTDIR}/${BACKUPDESC}/@home_${BACKUPDESC}"
btrfs subvolume snapshot /var/lib/machines "${SNAPSHOTDIR}/${BACKUPDESC}/machines_${BACKUPDESC}"

#
# Unmount
#
cd ~
umount "${TEMPMOUNTDIR}"
