#!/bin/bash -e

# Script for taking snapshots and storing these on a network share (compressed)
# Usage: SnapshotRemote.sh SnapshotName
#
# In oder to adapt this script for other installations:
# 1. Change the variables to fit for you installations
# 2. Check for the btrfs snapshots after installation (btrfs subvolume list /)
# 		If there are other snapshots than @, @home and /var/lib/machines, you have to change the handling of the snapshots further below

errorecho() { cat <<< "$@" 1>&2; }

# Variables
TEMPMOUNTDIR="/mnt/temp"						# Mount point of the btrfs root
SNAPSHOTDIR="${TEMPMOUNTDIR}/snapshots"			# Directory where snapshots are saved (usually under the btrs root)
DEVICE="/dev/sda6"								# The device (HDD) used
REMOTEBACKUPDIR="/mnt/Share/Backups"			# Directory where backups are stored (this has to be a already mounted network share)
BACKUPDESC=$1

#
# Check if parameter given
#
if [ $# != "1" ]
then
    errorecho "ERROR: No backup name to given!"
	errorecho "Usage: RemoteBackup.sh BackupName"
    exit 1
fi

#
# Check for root
#
if [ "$(id -u)" != "0" ]
then
	errorecho "ERROR: This script has to be run as root!"
	exit 1
fi

#
# Check if backup dir is available, i.e. Windows share is mounted
#
if [ ! -e "${REMOTEBACKUPDIR}" ]
then
	errorecho "ERROR: Cannot access remote backup directory ${REMOTEBACKUPDIR}!"
	errorecho "Check if the Windows share is mounted."
	exit 1
fi

#
# Check if backup exists already
#
if [[ -e "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_root_${BACKUPDESC}.tar.gz" || -e "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_home_${BACKUPDESC}.tar.gz" || -e "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_machines_${BACKUPDESC}.tar.gz" ]]
then
	errorecho "ERROR: Backup ${BACKUPDESC} already exists!"
	cd ~
	umount "${TEMPMOUNTDIR}"
	exit 1
fi

#
# Check if all dirs already exist
#
if [ ! -e "${SNAPSHOTDIR}" ]
then
	mkdir -p "${SNAPSHOTDIR}"
fi

#
# Mount device
#
mount "${DEVICE}" "${TEMPMOUNTDIR}"
cd "${TEMPMOUNTDIR}"
sync

#
# Check if all snapshot dir already exists
#
if [ ! -e "${SNAPSHOTDIR}" ]
then
	mkdir -p "${SNAPSHOTDIR}"
fi

#
# Create snapshots
#
btrfs subvolume snapshot "${TEMPMOUNTDIR}/@" "${SNAPSHOTDIR}/@_temp"
btrfs subvolume snapshot "${TEMPMOUNTDIR}/@home" "${SNAPSHOTDIR}/@home_temp"
btrfs subvolume snapshot /var/lib/machines "${SNAPSHOTDIR}/machines_temp"

#
# Tar
#
mkdir "${REMOTEBACKUPDIR}/${BACKUPDESC}"
tar cpzf "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_root_${BACKUPDESC}.tar.gz" "${SNAPSHOTDIR}/@_temp"
tar cpzf "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_home_${BACKUPDESC}.tar.gz" "${SNAPSHOTDIR}/@home_temp"
tar cpzf "${REMOTEBACKUPDIR}/${BACKUPDESC}/backup_machines_${BACKUPDESC}.tar.gz" "${SNAPSHOTDIR}/machines_temp"

#
# Delete snapshots
#
btrfs subvolume delete "${SNAPSHOTDIR}/machines_temp"
btrfs subvolume delete "${SNAPSHOTDIR}/@_temp"
btrfs subvolume delete "${SNAPSHOTDIR}/@home_temp"

#
# Unmount
#
cd ~
umount "${TEMPMOUNTDIR}"